(function (Drupal) {

  Drupal.smart_content.SmartContentManager = Drupal.smart_content.SmartContentManager || {};
  Drupal.smart_content.SmartContentManager.plugin = Drupal.smart_content.SmartContentManager.plugin || {};
  Drupal.smart_content.SmartContentManager.plugin.Field = Drupal.smart_content.SmartContentManager.plugin.Field || {};
  Drupal.smart_content.SmartContentManager.plugin.Field.demandbaseSmartCondition = {
    process: function (Field) {
      let pluginId = Field.pluginId.split(':');
      if (window.Demandbase.IP.CompanyProfile
        && typeof pluginId[1] !== 'undefined'
        && typeof window.Demandbase.IP.CompanyProfile[pluginId[1]] !== 'undefined') {
        let value = window.Demandbase.IP.CompanyProfile[pluginId[1]];
        Field.complete(value, true);
      }
      else {
        Field.complete(null, true);
      }
    },

    init: function (Field) {
      // pluginId[0] = base plugin ID.
      // pluginId[1] = Demandbase field key.
      let pluginId = Field.pluginId.split(':');
      if (pluginId[0] !== 'demandbase' || pluginId[1] == 'undefined') {
        return;
      }
      Field.claim();

      if(!Drupal.smart_content.storage.isExpired('demandbase')) {
        let values = Drupal.smart_content.storage.getValue('demandbase').value;
        let result = values.hasOwnProperty(pluginId[1]) ? values[pluginId[1]] : null;

        Field.complete(result, true);
        // Call process field either way, so we can store the value.
        Drupal.smart_content.DemandBase.processField(Field);
      }
      else {
        Drupal.smart_content.DemandBase.processField(Field);
      }
    }
  }

  Drupal.smart_content.DemandBase = Drupal.smart_content.DemandBase || {
    processing: false,
    completed: false,
    queue: [],
    process: function () {
      if(!this.processing) {
        this.processing = true;
        let attempts = 0;
        if(window.Demandbase && window.Demandbase.IP && window.Demandbase.IP.CompanyProfile && window.Demandbase.IP.CompanyProfile.ip) {
          this.completed = true;
          Drupal.smart_content.storage.setValue('demandbase', window.Demandbase.IP.CompanyProfile);
        }
        else {
          let processingInterval = setInterval(function (obj) {
            attempts++;
            if(window.Demandbase && window.Demandbase.IP && window.Demandbase.IP.CompanyProfile && window.Demandbase.IP.CompanyProfile.ip) {
              obj.completed = true;
              Drupal.smart_content.storage.setValue('demandbase', window.Demandbase.IP.CompanyProfile);
              clearInterval(processingInterval);
              obj.processQueue();
            }
            if (attempts >= 20) {
              obj.completed = true;
              clearInterval(processingInterval);
              obj.processQueue();
            }
          }, 100, this);
        }

      }
    },
    processQueue: function () {
      if(this.completed) {
        for(var i = 0; i < this.queue.length; i++ ) {
          this.processField(this.queue[i]);
          this.queue.splice(i, 1);
          i--;
        }
      }
    },
    processField: function (Field) {
      if(!this.processing) {
        this.process();
      }
      if(this.completed) {
        if(!Field.processed) {
          Drupal.smart_content.SmartContentManager.plugin.Field.demandbaseSmartCondition.process(Field);
        }
      }
      else {
        this.queue.push(Field);
      }
    }
  }

}(Drupal));
